#!/usr/data/php
<?php namespace MomentCore{

    /**
     * 信号传递组件
     */
    class Signal extends \Error{

        /**
         * @var mixed 信号传递的内容
         */
        protected $content;

        public function __construct(mixed $data = null) {
            $this -> content = $data;
        }

        public function getData(){
            return $this -> content;
        }

        public function __toString():string{
            return 'SIGNAL';
        }
    }
    
    class Promise{
        private $resolve = [];
        private $reject = [];
        private $finally = [];

        /**
         * @var int 存储执行状态值
         */
        public $status = self::TIMING;

        /**
         * @var mixin Promise返回值，用于应对提前兑现现象
         */
        private $temp;

        const TIMING = 0;
        const RESOLVE = 2;
        const REJECT = 3;
        const UNREADED = 1;

        /**
         * 通过迭代器实现await方法
         * 只能在顶层使用，例如
         * function aaa(){ yield async_func() }
         * function b(){ yield Promise::await(aaa()) }
         * Promise::await(b()) -> then(...)
         * 
         * @param mixed 异步内容，允许函数、函数返回值和Promise
         * @param ?callable 当获取内容前的回调（初次不会触发）
         */
        static function await(mixed $async,?callable $onext = null){
            /**
             * @var callable next
             */
            $next = null;
            
            if(is_callable($async))
                $async = $async();
            
            $return = new Promise($rs,$rj);

            $next = function() use ($async,&$next,$rs,$rj,$onext){
                // 返回的数据
                $await = $async -> current();
                // 回调函数
                $nexts = function($dat) use ($async,$rj,$next){
                    try{
                        $async -> send($dat);
                    }catch(\Throwable $e){
                        return $rj($e);
                    }
                    $next();
                };
                // 调用事件
                if(\is_callable($onext)) $onext($await);
                // 完成：许诺
                if(!\is_object($await) && !$async -> valid())
                    $rs($async -> getReturn());
                // 传递Promise
                elseif($await instanceof Promise)
                    $await -> then($nexts);
                // 传递Generator
                elseif($await instanceof \Generator)
                    self::await($await) -> then($nexts);
                // 没有匹配：依旧直接传递
                else
                    $nexts($await);
            };

            if($async instanceof \Generator)
                $next();
            elseif($async instanceof Promise)
                return $async;
            else
                $rs($async);

            return $return;
        }

        /**
         * 像Javascript一样创建新Promise
         * @param callable 自执行函数
         */
        static function new(callable $func){
            $return = new Promise($rs,$rj);
            $func($rs,$rj);
            return $return;
        }

        /**
         * 创建一个Promise，使用引用方式传递回调
         * @param ?callable 解决时的回调
         * @param ?callable 错误时的回调
         */
        public function __construct(mixed &$rs,mixed &$rj) {
            $finish = function(){
                foreach ($this -> finally as $finally) try{
                    \call_user_func($finally);
                }catch(\Throwable){}
            };
            $rs = function($result) use ($finish){
                $finish();
                if($this -> resolve){
                    foreach ($this -> resolve as $rs_handle)
                        \call_user_func($rs_handle,$result);
                    $this -> status = self::RESOLVE;
                }else{
                    $this -> temp = $result;
                    $this -> status = self::UNREADED;
                }
            };
            $rj = function($result) use ($finish){
                $finish();
                if(count($this -> reject) == 0)
                    if($result instanceof \Throwable)
                        throw $result;
                    else 
                        throw new \Error((string)$result,11);
                foreach ($this -> reject as $rj_handle)
                    call_user_func($rj_handle,$result);
                $this -> status = self::REJECT;
            };
        }

        /**
         * Promise兑现时的回调
         * 允许提前兑现现象时依旧有输出
         * @param callable 回调
         */
        public function then(callable $cb){
            if($this -> status == self::UNREADED){
                $cb($this -> temp);
                $this -> status = self::RESOLVE;
                $this -> temp = null;
            }
            $this -> resolve[] = $cb;
            return $this;
        }

        /**
         * Promise错误时的回调
         * 如果没有设置任何回调，将会直接变成全局错误
         * @param callable 回调
         */
        public function catch(callable $cb){
            @$this -> reject[] = $cb;
            return $this;
        }

        /**
         * Promise 完成时的回调，无论是否错误
         * @param callable 回调
         */
        public function finally(callable $cb){
            if($this -> status == self::TIMING)
                $this -> finally[] = $cb;
            else $cb();
            return $this;
        }
    }

    class Core{

        const FONT_BOLD = "\033[1m";
        const FONT_DARK = "\033[2m";
        const FONT_UNDERLINE = "\033[4m";
        const FONT_BLINK = "\033[5m";
        const FONT_INVERT = "\033[7m";
        const FONT_HIDE = "\033[8m";

        const FONT_UNSET_BOLD = "\033[21m";
        const FONT_UNSET_DARK = "\033[22m";
        const FONT_UNSET_UNDERLINE = "\033[24m";
        const FONT_UNSET_BLINK = "\033[25m";
        const FONT_UNSET_INVERT = "\033[27m";
        const FONT_UNSET_HIDE = "\033[28m";
        const FONT_UNSET_ALL = "\033[0m";

        const COLOR_BLACK = "\033[30m";
        const COLOR_RED = "\033[31m";
        const COLOR_GREEN = "\033[32m";
        const COLOR_YELLOW = "\033[33m";
        const COLOR_BLUE = "\033[34m";
        const COLOR_PINK = "\033[35m";
        const COLOR_CRAN = "\033[36m";
        const COLOR_WHITE = "\033[97m";
        const COLOR_GRAY = "\033[90m";
        const COLOR_GRAY_LIGHT = "\033[37m";
        const COLOR_RED_LIGHT = "\033[91m";
        const COLOR_GREEN_LIGHT = "\033[92m";
        const COLOR_YELLOW_LIGHT = "\033[93m";
        const COLOR_BLUE_LIGHT = "\033[94m";
        const COLOR_PINK_LIGHT = "\033[95m";
        const COLOR_CRAN_LIGHT = "\033[96m";
        
        const COLOR_UNSET = "\033[39m";

        const BG_BLACK = "\033[40m";
        const BG_RED = "\033[41m";
        const BG_GREEN = "\033[42m";
        const BG_YELLOW = "\033[43m";
        const BG_BLUE = "\033[44m";
        const BG_PINK = "\033[45m";
        const BG_CRAN = "\033[46m";
        const BG_GRAY_LIGHT = "\033[47m";
        const BG_GRAY = "\033[100m";
        const BGT_RED_LIGH = "\033[101m";
        const BG_GREEN_LIGHT = "\033[102m";
        const BG_YELLOW_LIGHT = "\033[103m";
        const BG_BLUE_LIGHT = "\033[104m";
        const BG_PINK_LIGHT= "\033[105m";
        const BG_CRAN_LIGHT = "\033[106m"; 
        const BG_WHITE = "\033[107m";

        const BG_UNSET = "\033[49m";
        
        static $server = [];
        static $client = [];
        static $handler = [];
        static $onfree = [];
        static $alias = [];

        /**
         * @var array Option请求的返回内容
         */
        static $option = [
            'DAV' => '1',
            'Method' => 'GET,POST,PUT,DELETE'
        ];

        const READ_BUFFER_SIZE = 64 * 1024;
        const WRITE_BUFFER_SIZE = 64 * 1024;
        const CHUNK_SIZE = 4 * 1024;
        const KEEPALIVE_TIMEOUT = 1 * 60;
        const EV_DELAY = .002 * 1000;

        /**
         * 执行PHP脚本并将输出重定向到HTTPClient
         * @param string 文件位置
         * @param HttpHandle 相对于的Handler
         * @param array 传入脚本的参数
         */
        static function run(string $path,HttpHandle $handle,array $param = []){
            if(!\is_file($path))
                throw new \Error("File not exists");

            // if(
            //     basename(__FILE__) == basename($path)
            //     && yield self::fget(__FILE__) == yield self::fget($path)
            // )
            //     return $handle -> finish(400,'Self couldnot be executed');
            
            // 优化变量
            $GLOBALS['__HANDLE__'] = $handle;
            \array_unshift($param,$handle);

            // 获取入口函数
            try{
                return Promise::await(call_user_func(include($path),$handle),function () use ($handle) {
                    $GLOBALS['__HANDLE__'] = $handle;
                })->finally(function () use ($handle) {
                    @ob_flush();
                    $handle -> finish();
                })->catch(function ($e) use ($handle){
                    $handle -> finish(500, self::parseError($e));
                });
            }catch(\ParseError $e){
                $handle -> finish(500,'Bad syntax: ' . $e -> getMessage());
            }catch(\Throwable $e){
                $handle -> finish(500, self::parseError($e));
            }
        }

        /**
         * 解析错误调用，屏蔽此文件的函数调用
         * 可以使用以下代码装饰
         * ```css
         * div.__error {
         *    position: fixed;
         *    top: 50vh;
         *    left: 50vw;
         *    width: 60vw;
         *    transform: translate(-50%, -50%);
         *    padding: 1.5rem;
         *    background-color: #e17272;
         *    border-left: solid .6rem #34e0e6;
         *    color: white;
         *    box-shadow: #c7bebe 0 0 1rem;
         * }
         * ```
         */
        static function parseError(\Throwable $data){
            $tmp = "<div class=\"__error\">Uncaught <b>".get_class($data)."</b> : ".$data->getMessage();
            foreach (array_reverse($data -> getTrace()) as $trace)
                if(__FILE__ !== @$trace["file"] && !str_starts_with($trace['function'],'MomentCore\\'))
                    $tmp .= "<div style=\"padding-left: 2em;\"><b>at</b> {$trace["class"]}<b>{$trace["type"]}{$trace["function"]}</b>:{$trace["line"]} <span style=\"color: gray;\">({$trace["file"]})</span></div>";
            return $tmp . "</div>";
        }

        /**
         * 使用异步IO打开一个文件
         * @param string 文件名
         * @param string 打开模式，与PHP的打开参数一致
         */
        static function open(string $file,string $fmode = 'r'){
            $f = @fopen($file,$fmode);
            if(!$f) throw new \Error("Failed to open file:" . error_get_last()['message'],1);
            $handle = new FileHandle($f,$file);
            $handle -> id = self::_add($f,$handle);
            return $handle;
        }

        /**
         * 读取一个文件
         */
        static function fget(string $fname){
            $file = self::open($fname);
            $temp = '';
            while(!$file -> end())
                $temp .= yield $file -> read();
            return $temp;
        }

        static function fetch(string $url,array $option,mixed &$header){
            $prom = new Promise($rs,$rj);
            Promise::await(function() use (&$url,&$option,&$header,$rs,$rj){
                $data = parse_url($url);
                $is_http = strtolower($data['scheme']) == 'http';
                $host = ($is_http ? 'tcp://' : 'tls://')
                    . $data['host'] 
                    . ':' . ($data['port'] ?? ($is_http ? 80 : 443));
                $uri = ($data['path'] ?? '/') . ($data['query'] ? '?' . $data['query'] : '');
                $option = array_merge($option,[
                    'method' => 'GET',
                    'header' => [
                        'User-Agent' => 'MomentPHP',
                        'Host' => $data['host'],
                        'Connection' => 'close'
                    ],
                    'forceRead' => false,
                    'redirect' => true,
                    'connect_timeout' => 10.0,
                    'timeout' => 10
                ]);

                $client = stream_socket_client($host,$ec,$em,$option['connect_timeout'],STREAM_CLIENT_ASYNC_CONNECT,stream_context_create([
                    'tcp' => [
                        'tcp_nodelay' => true
                    ],
                    'ssl' => [
                        'verify_peer' => false,
                        'verify_peer_name' => false
                    ]
                ]));
                if(false === $client)
                    return $rj(new \Error("Fetch failed($ec): $em"));
                $handle = new TcpHandle($client,$host);
                $handle -> timeout = $option['timeout'];
                $handle -> ontimeout = fn() => 
                    $rj(new \Error('Timeout'));
                self::_add($client,$handle);

                // 写请求头
                $handle -> write("{$option['method']} $uri HTTP/1.1\r\n");
                $handle -> write("Connection: close\r\n");
                foreach ($option['header'] as $key => $value)
                    $handle -> write("$key: $value\r\n");
                $handle -> write("\r\n");

                // 读取第一行
                list($header['_http_version'],$status,$header['_status_text']) = explode(' ',yield $handle -> readline(),3);
                $header['_status'] = $status = (int)$status;

                // 读取Header
                while(true)
                    if(!preg_match(
                        '/^\s*([a-z-_]+)\s*:\s*(.+)\s*$/i',
                        yield $handle -> readline(),$match
                    )) break;
                    else
                        $header[strtolower($match[1])] = $match[2];

                // 重定向检测
                if($option['redirect'] && ($status == 301 || $status == 302 || $status == 307)){
                    if($header['location'])
                        $rs(yield Promise::await(yield self::fetch($header['location'],$option,$header = [])));
                    else
                        $rj(new \Error('FetchError: illgal redirect'));
                }else
                    $rs($handle);
            });
            return $prom;
        }

        static function fetchTo(string $url,Handler $to,array $option){
            $h = yield self::fetch($url,$option,$_h = []);
            if(is_numeric($_h['content-length']))
                yield $h -> pipeTo($to,(int)$_h['content-length']);
            else
                yield $h -> pipeTo($to);
        }

        static function fetchText(string $url,array $option,mixed &$header){
            $handle = yield self::fetch($url,$option,$header);
            if(is_numeric($header['content-length']))
                return yield $handle -> read((int)$header['content-length']);
            else
                return yield $handle -> read();
        }

        /**
         * 向终端写入内容，而不是客户端
         */
        static function log(string $data,bool $wrap = true){
            fwrite(STDOUT,preg_replace_callback('/\\{\\s*(-|\\/)?\\s*([a-z_]+)?\\s*}/i',function(array $match){
                if($match[1] == '/')
                    return self::FONT_UNSET_ALL;
                $type = strtoupper($match[2]);
                if($match[1] == '-')
                    $type = $type . '_LIGHT';
                try{
                    return constant("self::$type");
                }catch(\Throwable){
                    return "{$type}";
                }
            },$data) . self::FONT_UNSET_ALL);
            if($wrap) fwrite(STDOUT,PHP_EOL);
        }

        static function dump(mixed $obj,string $obj_indent = ""){
            switch(strtolower(gettype($obj))){
                case "boolean":
                    $obj = $obj ? ' {color_green}true{/} ' : '{color_red}false{/}';
                break;

                case 'integer':
                case 'double':
                case 'float':
                    $obj = "{color_green} $obj {/}";
                break;

                case 'object':
                    $start = "Object {\n";
                    $end = "}";
                    $obj = (array)$obj;
                case 'array':
                    self::log(@$start ?? "Array [");
                    foreach ($obj as $key => $value){
                        self::log("$obj_indent\t{color_cran}$key{/} => ",false);
                        self::dump($value,$obj_indent . "\t");
                    }
                    self::log( $obj_indent . (@$end ?? "]"));
                return;
                    
                case 'resource':
                    $obj = '{color_green} Resource {/}';
                break;

                case 'resource (closed)':
                    $obj = '{color_red} !Resource {/}';
                break;

                case 'string':
                    $obj = " \"{color_pink}$obj{/}\" ";

                case 'null':
                    $obj = ' {color_gray}NULL{/} ';
                break;
            }
            Core::log($obj);
        }

        /**
         * 单次延时执行(取决于EventLoop时长)
         * 返回Promise，直接使用 `Core::nextTick()->then(...)` 即可
         * @param ?mixed 兑现时返回的参数
         */
        static function nextTick(mixed $param = null){
            return Promise::new(function($rs) use ($param){
                self::$onfree[] = function() use ($rs,$param){
                    $rs($param);
                    throw new \Exception("");
                };
            });
        }

        /**
         * 向EventLoop加入一个对象
         * 可以是这些
         *  - `fopen()` 等打开的文件IO
         *  - `fsockopen()` 等打开的网络IO
         *  - `proc_open()` 等打开的进程输入输出IO
         * 
         * @param mixed Resource对象
         * @param object 对应的Handler，必须具有`_read` `_write`函数
         */
        static function _add(mixed $rs,object $data){
            if(!is_resource($rs)) return false;
            $id = uniqid();
            self::$client[$id] = $rs;
            self::$handler[$id] = $data;

            stream_set_blocking($rs, false);
            stream_set_read_buffer($rs, self::READ_BUFFER_SIZE);
            stream_set_write_buffer($rs, self::WRITE_BUFFER_SIZE);
            stream_set_chunk_size($rs, self::CHUNK_SIZE);

            return $id;
        }

        /**
         * 接受一个客户端
         */
        static function __accept($client,?string $name){
            try {
                if (!$client) return false;

                $handle = new HttpHandle($client, $name);
                $handle -> id = self::_add($client,$handle);

                Promise::await(function() use ($handle){
                    while(true){
                        yield $handle -> accept();

                        if($handle -> path[0] != '/')
                            return $handle -> error(400,"Bad URL");
                        if(\strpos("..",$handle -> path) > 0)
                            return $handle -> finish(301,"",[
                                "Location" => \preg_replace('/\\/([\\w\\W^/]\\/)?\\.\\.\\//','',$handle -> path)
                            ]);

                            /**
                             * @var string 文件路径
                             */
                            $path = CONFIG['rounger']['root'] . $handle -> path;
                            /**
                             * @var string 文件名
                             */
                            $ext = \strtolower(@\end(\explode(".",basename($handle -> path))));
                            
                            // alias 支持
                            if(array_key_exists($handle -> path,self::$alias))
                                $path = self::$alias[$handle -> path];
                            
                            /**
                             * @var bool 文件是否存在
                             */
                            $exist = \file_exists($path);
                            
                            // OPTIONS请求
                            if($handle -> method == 'OPTIONS'){
                                $handle -> finish(204,'',self::$option);
                            // 是可执行PHP文件
                            }elseif($exist && \in_array($ext,CONFIG['rounger']['execable'])){
                                Core::run($path,$handle);
                            // HTTP FileServer
                            }elseif($handle -> method == 'GET' || $handle -> method == 'HEAD'){
                                // 文件夹服务器
                                if(!$exist)
                                    $handle -> finish(404,'Not found');
                                // 使用函数便于Escape
                                elseif(\is_dir($path)) yield (function() use ($handle,$path){
                                    if(\substr($path,-1) != '/'){
                                        $path .= '/';
                                        $handle -> header('Content-Location',$path);
                                    }
                                    foreach (CONFIG['rounger']["index"] as $index)
                                        if(\file_exists($path . $index))
                                            if(\in_array(
                                                \strtolower(@end(explode(".",$index))),
                                                CONFIG['rounger']['execable'])
                                            )
                                                return Core::run($path . $index,$handle);
                                            else
                                                return $handle -> file($path . $index);
                                        // AutoIndex
                                        if(CONFIG['rounger']['autoindex'])
                                            $handle -> listDir($path,$handle -> path != '/');
                                        else
                                            $handle -> finish(403,'Is a dir');
                                })();
                                // 文件服务
                                else yield $handle -> file($path);
                            // WebDAV服务
                            }else{
                                yield $handle -> webdav();
                            }
                    }
                });
            } catch (\Throwable $e) {
                self::log("{color_red}E{/} Failed to deal client(line " . $e -> getLine() . "):" . $e->getMessage());
            }
        }

        /**
         * 主函数
         */
        static function start(){

            if(count(self::$server) == 0)
                throw new \Error('At least 1 addr should be specificed.');

            // 初始化OB缓冲区
            @\ob_end_clean();
            \ob_start(function(string $data){
                /**
                 * @var HttpHandle
                 */
                $handle = $GLOBALS['__HANDLE__'];
                if($handle && $handle -> state() == HttpHandle::TIMING)
                    $handle -> write($data);
                return '';
            },self::CHUNK_SIZE,PHP_OUTPUT_HANDLER_CLEANABLE|PHP_OUTPUT_HANDLER_FLUSHABLE);

            while(true){
                foreach (self::$handler as $i => $handle)
                    if($handle -> closed()){
                        unset(self::$client[$i]);
                        unset(self::$handler[$i]);
                        if(CONFIG['behavior']['displayWarning'])
                            Core::log("{color_blue}I{/} Socket $i Closed");
                    }

                $readable = \array_merge(self::$client,self::$server); 
                $writable = self::$client;
                // $except = self::$server;
                $except = null;
                $changed = \stream_select($readable,$writable,$except,0);

                if($changed){
                    // 读取
                    foreach ($readable as $client => $value)
                        if(\is_string($client)) try{
                            self::$handler[$client]->_read();
                        }catch(\Throwable $e){
                            self::log("{color_red}E{/} Failed to Read Client(line " . $e -> getLine() . "):" . $e->getMessage());
                        }else{
                            self::__accept(stream_socket_accept($value, 0, $name),$name); 
                        }

                    // 写入
                    foreach ($writable as $client => $value) 
                        try{
                            self::$handler[$client]->_write();
                        }catch(\Throwable $e){
                            self::log("{color_red}E{/}Failed to WriteTo Client:" . $e->getMessage());
                        }
                }  
                // self::log('{color_blue}N{/}' . count($readable) . ' ' . count($writable));

                foreach(self::$onfree as $i => $func)
                    try{
                        $func();
                    }catch(Signal){
                        unset(self::$onfree[$i]);
                    }catch(\Throwable $e){
                        self::log("{color_red}E{/} " . $e -> getMessage() . " at " . $e -> getFile());
                    }

                usleep(self::EV_DELAY * 1000);
            }
        }

        /**
         * 创建一个监听服务器
         * @param string 绑定地址
         * @param object `stream_context_create`创建的context
         */
        static function bind(string $addr,?object $context = null) {

            if(!$context)
                $context = \stream_context_create([
                    'tcp' => [
                        'backlog'       => 4,
                        'so_reuseport'  => false,
                        'tcp_nodelay'   => true
                    ]
                ]);
            
            $ec = 0;
            $emsg = '';
            try{
                $socket = \stream_socket_server($addr,$ec,$emsg,context:$context);
            }catch(\Throwable $e){
                throw new \Error("Failed to bind port: " . $e->getMessage());
            }
            if(!$socket)
                return self::log("{color_yellow}W{/} Bind Failed($ec: $emsg)");

            Core::log("{color_green}I{/} Listen on $addr");

            return self::$server[] = $socket;
        }

    }

    abstract class Handler{
        /**
         * @var string ID
         */
        public $id;
        
        /**
         * @var resource 当前处理的Stream
         */
        protected $socket;

        /**
         * @var string|null 写入缓冲区
         */
        protected $write_buffer = '';

        /**
         * @var string 读取缓冲区
         */
        protected $read_buffer = '';

        /**
         * @var callable 达到数据量后传递的Promise
         */
        protected $reader;

        /**
         * @var callable 写入区空白（写入完成）后传递的Promise
         */
        protected $writed;

        /**
         * @var string 地址
         */
        readonly string $addr;

        /**
         * @var int 超时
         */
        protected $time = 0;

        /**
         * @var callable 用于限制读取，返回false表示无法读取
         */
        public $read_filter;

        /**
         * 创建Handler
         * 注意，不应该由用户自行创建。此函数由Eventloop自动调用
         */
        public function __construct($client,string $name) {
            $this -> socket = $client;
            $this -> addr = $name;
            $this -> time = time();
        }

        /**
         * 向对象写入内容
         * 使用非阻塞IO，无需担心写入卡死
         * @param string 写入的数据
         */
        public function write(string $data){
            $this -> write_buffer .= $data;
        }

        private function _reader(callable $cb,?callable $filter = null){
            $this -> reader = $cb;
            $data = $this -> read_buffer;
            $this -> read_buffer = '';
            
            $this -> read_filter = $filter ?? fn () => 
                $this -> read_buffer <= Core::READ_BUFFER_SIZE;

            $cb($data);
        }

        /**
         * 读取指定大小内容
         * @param int 读取长度，-1表示读取缓冲区内所有内容
         */
        public function read(int $byte = -1){
            $prom = new Promise($rs,$rj);

            // 无限制
            if($byte <= 0){
                $rs($this -> read_buffer);
                $this -> read_buffer = '';
            }else{
                if($this -> reader)
                    throw new \Error("Queue: readIO is busy");
                $this -> _reader(function(string $data = '') use ($byte,$rs){
                    $len = strlen($data);
                    if($len >= $byte){
                        $data = substr($data ,0 ,$byte);
                        $this -> read_buffer = substr($data, $byte);
                        $this -> reader = null;
                        $rs($data);
                    }
                });
            }
            
            return $prom;
        }

        static function _trim(string $str,bool $left = true){
            if($str == '') return $str;
            if($left)
                if($str[0] == "\r" || $str[0] == "\n")
                    return substr($str,1);
                else return $str;
            else
                if($str[$len = (strlen($str) -1)] == "\r" || $str[$len] == "\n")
                    return substr($str,0,$len);
                else return $str;
        }

        /**
         * 读取一行文本，同时兼容Windows与Linux
         */
        public function readline(){
            if($this -> reader)
                throw new \Error("Queue: readIO is busy");

            $prom = new Promise($rs,$rj);

            $this -> _reader(function (string $data) use ($rs) {
                $pos = strpos($data, "\n");
                if (false === $pos) $pos = strpos($data, "\r");
                if ($pos !== false) {
                    $this->reader = null;
                    $this->read_buffer = self::_trim(substr($data, $pos + 1));
                    $rs(self::_trim(substr($data, 0, $pos), false));
                }
            });

            return $prom;
        }

        public function pipeTo(Handler $handle,int $expect = -1):Promise{
            if($this -> reader)
                throw new \Error("Queue: readIO is busy");
            $promise = new Promise($rs,$rj);
            $readed = 0;
            $this -> _reader(function(string $data) use ($handle,$expect,&$readed,$rs){
                // 目标已关闭
                if($handle -> closed()){
                    $this -> read_buffer .= $data;
                    $this -> reader = NULL;
                    $rs(null);
                // 存在读取限制
                }elseif($expect > 0){
                    $require = $expect - $readed;
                    $buflen = strlen($data);
                    // 过多
                    if($buflen >= $require){
                        $handle -> write(substr($data,0,$require));
                        $this -> read_buffer .= substr($data,$require);
                        $this -> reader = NULL;
                        $rs(null);
                    // 不足
                    }else{
                        $handle -> write($data);
                    }
                // 不存在限制
                }else{
                    $handle -> write($data);
                }
            // 加入等待机制
            },function() use ($handle){
                return $handle -> bufferState()['write'] <= Core::WRITE_BUFFER_SIZE;
            });
            return $promise;
        }

        /**
         * 此函式应该由EventLoop调用，表示可写
         */
        public function _write(){
            if(strlen($this -> write_buffer) >= Core::WRITE_BUFFER_SIZE*2)
                return;
            if($this -> write_buffer){
                $write = @fwrite($this -> socket,$this -> write_buffer);
                $this -> write_buffer = substr($this -> write_buffer,$write);
                $this -> time = time();
            }elseif($this -> writed)
                call_user_func($this -> writed);
        }

        /**
         * 此函式应该由EventLoop调用，表示可读
         */
        public function _read(){
            if($this -> read_filter && $this -> reader
                && call_user_func($this -> read_filter) !== false
            ) call_user_func($this -> reader,fread($this -> socket, Core::READ_BUFFER_SIZE));
            $this -> time = time();
        }

        public function bufferState(){
            return [
                'read' => strlen($this -> read_buffer),
                'write' => strlen($this -> write_buffer)
            ];
        }

        /**
         * 关闭Stream
         */
        abstract function close():void;
        
        /**
         * 获取是否已经关闭
         */
        abstract function closed():bool;
    }

    /**
     * 处理文件IO的类
     * 支持异步
     */
    class FileHandle extends Handler{
        /**
         * 移动起始点
         */
        function seek(int $len){
            fseek($this -> socket,$len,SEEK_CUR);
        }

        /**
         * 获取是否已经关闭
         */
        public function closed():bool{
            return !is_resource($this -> socket);
        }

        /**
         * 获取文件信息
         */
        public function stat(){
            return fstat($this -> socket);
        }

        /**
         * 获取是否读完
         */
        public function end(){
            return feof($this -> socket);
        }

        /**
         * 关闭IO
         */
        public function close():void{
            fclose($this -> socket);
        }
    }

    class TcpHandle extends Handler{

        /**
         * @var int 超时时间
         */
        public $timeout;

        /**
         * @var callable 超时回调
         */
        public $ontimeout;

        /**
         * 获取是否已经关闭
         */
        public function closed():bool{
            $timeout = time() - $this -> time > $this -> timeout;
            if($timeout && $this -> ontimeout)
                call_user_func($this -> ontimeout);
            return $timeout ||
                (feof($this -> socket) && !is_resource($this -> socket));
        }

        /**
         * 获取文件信息
         */
        public function stat(){
            return fstat($this -> socket);
        }

        /**
         * 获取是否读完
         */
        public function end(){
            return feof($this -> socket);
        }

        /**
         * 关闭IO
         */
        public function close():void{
            fclose($this -> socket);
        }
    }

    /**
     * 处理基础HTTP任务的类
     * 快速使用：bind([port])();
     */
    class HttpHandle extends Handler{
        /**
         * @var array 状态解释
         */
        static public $statusMap = [     // HTTP状态对应描述
            100 => 'Continue',
            101 => 'Switching Protocols',
            102 => 'Processing',
            200 => 'OK',
            201 => 'Created',
            202 => 'Accepted',
            203 => 'Non-Authoritative Information',
            204 => 'No Content',
            205 => 'Reset Content',
            206 => 'Partial Content',
            207 => 'Multi-Status',
            208 => 'Already Reported',
            300 => 'Multiple Choices',
            301 => 'Moved Permanently',
            302 => 'Found',
            303 => 'See Other',
            304 => 'Not Modified',
            307 => 'Temporary Redirect',
            308 => 'Permanent Redirect',
            400 => 'Bad Request',
            401 => 'Unauthorized',
            402 => 'Payment Required',
            403 => 'Forbidden',
            404 => 'Not Found',
            405 => 'Method Not Allowed',
            406 => 'Not Acceptable',
            407 => 'Proxy Authentication Required',
            408 => 'Request Timeout',
            409 => 'Conflict',
            410 => 'Gone',
            411 => 'Length Required',
            412 => 'Precondition Failed',
            413 => 'Payload Too Large',
            414 => 'URI Too Long',
            415 => 'Unsupported Media Type',
            416 => 'Range Not Satisfiable',
            417 => 'Expectation Failed',
            421 => 'Misdirected Request',
            422 => 'Unprocessable Entity',
            423 => 'Locked',
            424 => 'Failed Dependency',
            425 => 'Too Early',
            426 => 'Upgrade Required',
            428 => 'Precondition Required',
            429 => 'Too Many Requests',
            431 => 'Request Header Fields Too Large',
            451 => 'Unavailable For Legal Reasons',
            500 => 'Internal Server Error',
            501 => 'Not Implemented',
            502 => 'Bad Gateway',
            503 => 'Service Unavailable',
            504 => 'Gateway Timeout',
            505 => 'HTTP Version Not Supported',
            507 => 'Insufficient Storage',
            508 => 'Loop Detected',
            510 => 'Not Extended',
            511 => 'Network Authentication Required',
        ];

        /**
         * @var array 默认响应头
         */
        static $default = [
            'Server'        =>  'MomentPHP 3.1',
            'Content-Type'  =>  'text/html ;charset=UTF-8',
            'Connection'    =>  'keep-alive'
        ];

        /**
         * @var array MIME列表
         */
        static $mime = [
            'html' => 'text/html',
            'htm' => 'text/html',
            'shtml' => 'text/html',
            'css' => 'text/css',
            'xml' => 'text/xml',
            'gif' => 'image/gif',
            'jpeg' => 'image/jpeg',
            'jpg' => 'image/jpeg',
            'js' => 'application/javascript',
            'atom' => 'application/atom+xml',
            'rss' => 'application/rss+xml',
            'mml' => 'text/mathml',
            'txt' => 'text/plain',
            'jad' => 'text/vnd.sun.j2me.app-descriptor',
            'wml' => 'text/vnd.wap.wml',
            'htc' => 'text/x-component',
            'png' => 'image/png',
            'tif' => 'image/tiff',
            'tiff' => 'image/tiff',
            'wbmp' => 'image/vnd.wap.wbmp',
            'ico' => 'image/x-icon',
            'jng' => 'image/x-jng',
            'bmp' => 'image/x-ms-bmp',
            'svg' => 'image/svg+xml',
            'svgz' => 'image/svg+xml',
            'webp' => 'image/webp',
            'woff' => 'application/font-woff',
            'jar' => 'application/java-archive',
            'war' => 'application/java-archive',
            'ear' => 'application/java-archive',
            'json' => 'application/json',
            'hqx' => 'application/mac-binhex40',
            'doc' => 'application/msword',
            'pdf' => 'application/pdf',
            'ps' => 'application/postscript',
            'eps' => 'application/postscript',
            'ai' => 'application/postscript',
            'rtf' => 'application/rtf',
            'm3u8' => 'application/vnd.apple.mpegurl',
            'xls' => 'application/vnd.ms-excel',
            'eot' => 'application/vnd.ms-fontobject',
            'ppt' => 'application/vnd.ms-powerpoint',
            'wmlc' => 'application/vnd.wap.wmlc',
            'kml' => 'application/vnd.google-earth.kml+xml',
            'kmz' => 'application/vnd.google-earth.kmz',
            '7z' => 'application/x-7z-compressed',
            'cco' => 'application/x-cocoa',
            'jardiff' => 'application/x-java-archive-diff',
            'jnlp' => 'application/x-java-jnlp-file',
            'run' => 'application/x-makeself',
            'pl' => 'application/x-perl',
            'pm' => 'application/x-perl',
            'prc' => 'application/x-pilot',
            'pdb' => 'application/x-pilot',
            'rar' => 'application/x-rar-compressed',
            'rpm' => 'application/x-redhat-package-manager',
            'sea' => 'application/x-sea',
            'swf' => 'application/x-shockwave-flash',
            'sit' => 'application/x-stuffit',
            'tcl' => 'application/x-tcl',
            'tk' => 'application/x-tcl',
            'der' => 'application/x-x509-ca-cert',
            'pem' => 'application/x-x509-ca-cert',
            'crt' => 'application/x-x509-ca-cert',
            'xpi' => 'application/x-xpinstall',
            'xhtml' => 'application/xhtml+xml',
            'xspf' => 'application/xspf+xml',
            'zip' => 'application/zip',
            'bin' => 'application/octet-stream',
            'exe' => 'application/octet-stream',
            'dll' => 'application/octet-stream',
            'deb' => 'application/octet-stream',
            'dmg' => 'application/octet-stream',
            'iso' => 'application/octet-stream',
            'img' => 'application/octet-stream',
            'msi' => 'application/octet-stream',
            'msp' => 'application/octet-stream',
            'msm' => 'application/octet-stream',
            'docx' => 'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
            'xlsx' => 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
            'pptx' => 'application/vnd.openxmlformats-officedocument.presentationml.presentation',
            'mid' => 'audio/midi',
            'midi' => 'audio/midi',
            'kar' => 'audio/midi',
            'mp3' => 'audio/mpeg',
            'ogg' => 'audio/ogg',
            'm4a' => 'audio/x-m4a',
            'ra' => 'audio/x-realaudio',
            '3gpp' => 'video/3gpp',
            '3gp' => 'video/3gpp',
            'ts' => 'video/mp2t',
            'mp4' => 'video/mp4',
            'mpeg' => 'video/mpeg',
            'mpg' => 'video/mpeg',
            'mov' => 'video/quicktime',
            'webm' => 'video/webm',
            'flv' => 'video/x-flv',
            'm4v' => 'video/x-m4v',
            'mng' => 'video/x-mng',
            'asx' => 'video/x-ms-asf',
            'asf' => 'video/x-ms-asf',
            'wmv' => 'video/x-ms-wmv',
            'avi' => 'video/x-msvideo',
            'ttf' => 'font/ttf'
        ];

        /**
         * @var string autolist CSS文件
         */
        static $css = 'f,d,h {
                display: block;
                margin: .2rem .5rem;
            }
            h > c,h > a {
                font-weight: 400;
                font-size: .9rem;
            }
            h > a{
                padding-left: .5rem;
            }
            c, a {
                line-height: 1.5rem;
                box-sizing: border-box;
                display: inline-block;
            }
            a {
                width: 60%;
                word-break: break-all;
                white-space: nowrap;
                overflow: hidden;
                position: relative;
                text-overflow: ellipsis;
            }
            c:nth-child(even) {
                width: 15%;
                text-align: center;
            }
            c:nth-child(odd) {
                width: 25%;
                text-align: right;
                padding-right: 1rem;
            }
            a {
                text-decoration: none;
                color: inherit;
            }
            body {
                background-color: #F5F5F5;
                width: 90vw;
                max-width: 50rem;
                margin: auto;
                font-family: ui-sans-serif, system-ui, -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, "Noto Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
            }
            h2 {
                margin-bottom: 12px;
            }
            div {
                background-color: white;
                border-radius: .5rem;
                box-shadow: 0 .6rem 1rem -.3rem #00000017, 0 .3rem .5rem -.4rem #0000001a;
                font-size: .85rem;
                padding: .35rem 0;
            }
            span {
                font-size: .95rem;
                color: #787878;
                margin: .5rem;
            }
            f > a::before {
                content: url(\'data:image/svg+xml,<svg xmlns="http://www.w3.org/2000/svg" fill="aqua" viewBox="0 0 16 16"><path d="M14 4.5V14a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h5.5L14 4.5zm-3 0A1.5 1.5 0 0 1 9.5 3V1H4a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V4.5h-2z"/></svg>\');
            }
            d > a::before {
                content: url(\'data:image/svg+xml,<svg xmlns="http://www.w3.org/2000/svg" fill="aquamarine" viewBox="0 0 16 16"> <path d="M.54 3.87.5 3a2 2 0 0 1 2-2h3.672a2 2 0 0 1 1.414.586l.828.828A2 2 0 0 0 9.828 3h3.982a2 2 0 0 1 1.992 2.181l-.637 7A2 2 0 0 1 13.174 14H2.826a2 2 0 0 1-1.991-1.819l-.637-7a1.99 1.99 0 0 1 .342-1.31zM2.19 4a1 1 0 0 0-.996 1.09l.637 7a1 1 0 0 0 .995.91h10.348a1 1 0 0 0 .995-.91l.637-7A1 1 0 0 0 13.81 4H2.19zm4.69-1.707A1 1 0 0 0 6.172 2H2.5a1 1 0 0 0-1 .981l.006.139C1.72 3.042 1.95 3 2.19 3h5.396l-.707-.707z"/> </svg>\')
            }
            a::before {
                display: inline-block;
                width: 1em;
                margin: 0 .5em;
                transform: scale(1.5);
            }
        ';

        const DESTROYED = -1;
        const CREATED = 0;
        const WAITING = 1;
        const TIMING = 2;
        const CLOSED = 3;
        const WEBSOCKET = 4;

        /**
         * @var int 最大文件上传大小
         */
        static $maxUpload = 50*1024*1024; 

        /**
         * @var array SESSION存储器
         */
        static private $session = [];

        /**
         * @var string DAV根目录
         */
        static $dav_root = '.';

        /**
         * @var array 存储用户COOKIE
         */
        private $cookies = [];          // COOKIE设置需要很多行

        /**
         * @var int 请求状况
         */
        private $state = self::CREATED;

        /**
         * @var array 存储用户响应头
         */
        public $headers = [];

        /**
         * @var array 客户端请求头
         */
        public $requestHeader = [];

        /**
         * @var array 存储GET参数
         */
        public $param = [];

        /**
         * @var array|string 存储POST参数
         */
        public $post = null;

        /**
         * @var int 最终HTTP响应状态码
         */
        public $status = 200;

        /**
         * @var string 内容缓冲区
         */
        public $temp = '';

        /**
         * @var string HTTP访问路径
         */
        public $path;

        /**
         * @var string HTTP类型(GET/POST...)
         */
        public $method;

        /**
         * 读取SESSION
         * @param string 对方IP地址+COOKIE
         */
        static private function _getSession(string $uuid):mixed{
            if(!array_key_exists($uuid,self::$session)) return false;
            return self::$session [$uuid];
        }

        /**
         * 写入SESSION
         * @param string 对方IP地址+COOKIE
         */
        static private function _setSession(string $uuid , mixed $val):void{
            if(!array_key_exists($uuid,self::$session)) self::$session[$uuid] = [];
            @self::$session [$uuid] = $val;
        }

        /**
         * 解析真实IP(不带端口)
         * @param string 用户地址
         */
        static function getUrl(string $addr){
            $pos = strrpos($addr,':');
            return substr($addr,0,$pos);
        }

        /**
         * 关闭连接
         */
        public function close():void{
            $this -> writed = function(){
                stream_socket_shutdown($this -> socket,STREAM_SHUT_RDWR)
                    or fclose($this -> socket);
                $this -> writed = null;
            };
            $this -> state != self::DESTROYED;
        }

        /**
         * 获取是否已经关闭
         */
        public function closed():bool{
            return (time() - $this -> time >= Core::KEEPALIVE_TIMEOUT )
                || !is_resource($this -> socket) || feof($this -> socket);
        }

        /**
         * 获取Handle状况
         */
        public function state(){
            return $this -> state;
        }

        /**
         * 开始响应请求
         * 请使用`Promise::await()`以复用TCP连接
         */
        public function accept(){

            if($this -> closed())
                throw new \Error("Client has already closed.");

            if($this->state == self::WAITING || $this->state == self::WAITING)
                throw new \Error('The request is still running.');
            elseif($this -> state == self::WEBSOCKET)
                throw new \Error('Cannot accept requests from WebSocket Content');

            // 检测body是否清空
            try{
                yield $this -> parseEndParam();
            }catch(\Throwable){}

            // 重新定义
            $this -> state = self::CREATED;
            $this -> cookies = [];
            $this -> headers = [];
            $this -> param = [];
            $this -> requestHeader = [];
            $this -> status = 200;
            $this -> post = null;

            // 读取基础信息，若非3个参数、非GET、POST响应拦截
            $line = yield $this -> readline();
            @list($type,$path,$version) = explode(' ',$line,3);
            $type = strtoupper($type);
            if(!$type or !$path or !$version) {
                $this -> finish(400,'Bad header');
                return $this -> close();
            }

            // 基本处理URL参数，用查询"?"
            if(strpos($path,'?') != false){
                list($path,$param) = \explode('?',$path,2);
                $this->param = self::parseParam($param);
            }
            $this->path = $path = str_replace(
                ['/./', '//',   '\\'],
                ['/',   '/',    '/'],
                urldecode($path)
            );
            $this->method = $type;
            

            // 读取所有内容
            yield $this -> parserHeader(true);

            // 输出消息
            Core::log("{color_blue}C{/} $type({$this -> addr}) $path");
        }

        /**
         * 输出内容并结束请求
         * 
         * @param ?string 额外的数据
         * @param int HTTP状态码
         * @param array 额外添加的Header
         */
        public function finish(int $status = -1,string $content = '',array $header = []){
            if($this -> state != self::TIMING)
                throw new \Error('The request is not in TIMING state.');

            // 写HTTP响应头
            $header['Content-Length'] = strlen($content) + strlen($this->temp);
            if($status > 0) $this -> status = $status;      // HTTP状态码
            $this -> mkHeader(null,$header);

            if($this -> method != 'HEAD'){
                // 写入到缓冲区
                $this -> write_buffer .= $this -> temp;        // 缓冲区
                $this -> write_buffer .= $content;             // 写追加的内容
            }

            // 完成请求
            $this -> temp = '';                   // 清空body缓冲区
            $this -> state == self::CREATED;      // 使命结束
        }

        /**
         * 输出内容并关闭请求
         */
        public function error(int $status = 400,?string $content = 'Bad request',?array $header = []){
            $this -> finish($status,$content,$header);
            $this -> close();
        }

        /**
         * 设置响应状态码
         * @param int 状态码
         */
        public function status(int $http_status_code):HttpHandle{
            if($this -> state != self::TIMING)
                throw new \Error('The request is not in TIMING state.');
            $this->status = $http_status_code;
            return $this;
        }

        /**
         * 解析请求参数
         * @param string 参数字符串
         * @param ?string 分割符，如COOKIE是";"
         */
        private function parseParam(string $param,string $split = '&'):array{
            $tmp = [];
            foreach(\explode($split,$param) as $_) {
                list($name,$value) = \explode('=',$_,2);
                $tmp[urldecode(trim($name))]=\urldecode(trim($value));
            }
            return $tmp;
        }

        /**
         * 根据后缀名获取指定文件的MIME类型
         * @param string 文件名称
         */
        static function getMime(string $file):string{
            $ext = @end(explode('.',$file));             // 截取末尾扩展名
            return self::$mime[$ext] ?? 'application/octet-stream';
        }

        /**
         * 解析当前请求的请求头，如第一个参数是false将不保存到缓存，用于忽略
         * @param ?bool $save
         */
        private function parserHeader(bool $save = true) {
            $tmp = [];
            $this -> state = self::WAITING;
            while(true) {
                $str = trim(yield $this -> readline());
                if(trim($str) == "") break;
                @list($name,$value) = \explode(':',$str,2);
                $tmp [ \strtolower($name) ] = trim($value);
            }
            if($save === true) $this->requestHeader = $tmp;
            $this -> state = self::TIMING;
            return $tmp;
        }
        
        /**
         * 创建响应头
         * @param ?int 状态码
         * @param ?array 额外的Header
         */
        public function mkHeader(?int $status = null , $header = []):void {
            if($this -> state != self::TIMING)
                throw new \Error('The request is not in TIMING state.');
            if($this->headers === false)
                throw new \Error('Headers is not available.');
            $status = $status ?? $this->status;
            $desc = self::$statusMap[$status] ?? 'OK';
            $this -> write_buffer .= "HTTP/1.1 $status $desc\r\n";
            foreach(\array_merge(self::$default,$this->headers,$header,[
                'Date'=>\gmdate('D, d M Y H:i:s T')
            ]) as $n=>$v) $this -> write_buffer .= "$n: $v\r\n";
            foreach($this->cookies as $cookie)
                $this -> write_buffer .= "\r\nSet-Cookie: $cookie";
            $this -> write_buffer .= "\r\n";
            $this -> headers = false;
        }
        
        /**
         * 设置响应Header
         * @param string 名称
         * @param string 内容
         */
        public function header(string $name,string $value) {
            if($this -> state != self::TIMING)
                throw new \Error('The request is not in TIMING state.');
            $this->headers[$name] = $value;
            return $this;
        }

        /**
         * 获取已经设置的响应头，不存在返回NULL
         * @param string 响应头名称
         */
        
        public function getHeader(string $name):string|null {
            if($this -> state != self::TIMING)
                throw new \Error('The request is not in TIMING state.');
            return @$this->requestHeader[$name];
        }

        /**
         * (带r的表示request，请求标头。)
         * 获取客户端请求头，不存在返回NULL
         * @param string 请求头名称
         */
        public function getrHeader(string $name):string|null {
            if($this -> state != self::TIMING)
                throw new \Error('The request is not in TIMING state.');
            return @$this->requestHeader[$name];
        }

        /**
         * 解析请求头的末尾位置，一般是POST数据，不存在返回NULL
         */
        private function parseEndParam() {
            if($this->method != 'POST' || null != $this -> post);
                return;
            $this -> state = self::WAITING;
            $len = $this->getrHeader('content-length');// 最终数据大小
            if($len > self::$maxUpload)
                return $this -> error(413,'上传内容太大');
            elseif($len <= 0)
                return;
            $dat = yield $this -> read($len);          // 传递数据
            if(substr($this->getrHeader('content-type'),0,33) == 'application/x-www-form-urlencoded'){
                // 以 名=值&...传递
                $this->post = yield $this->parseParam($dat);
            }elseif(substr($this->getrHeader('content-type'),0,19) == 'multipart/form-data'){
                // 上传文件
                $split = '--'.\substr($this->getrHeader('content-type'),strpos($this->getrHeader('content-type'),'boundary=') + 9);// 读取分界符
                foreach(explode($split.PHP_EOL,$dat) as $f)
                    @$this->post['_FILE'][] = new FileObject($f);
            } else {
                // 不清除，直接保存
                $this->post = $dat;
            }
            $this -> state = self::TIMING;
        }

        /**
         * 获取一个URL参数，如果是POST的数据请第三个参数为true
         * @param string $param_name
         * @param bool 是否为POST请求
         * @return null|string
         */
        public function getParam(string $param,bool $type_post = false) {
            if($this -> state != self::TIMING)
                throw new \Error('The request is not in TIMING state.');
            return $type_post? @$this->post[$param]: @$this->param[$param];
        }

        /**
         * 获取一个URL Cookie，客户端传来的。
         * 【注意】这并不安全，如果想安全使用getSession、setSession
         * 
         * @param string $get_cookie_name
         */
        public function getCookie(string $cookie):string|null {
            if($this -> state != self::TIMING)
                throw new \Error('The request is not in TIMING state.');
            if(is_null($this->getrHeader('cookie'))) return null;
            return @$this->parseParam($this->getrHeader('cookie'),';')[$cookie];
        }

        /**
         * 写一个COOKIE
         * 
         * @param string $cookie_name
         * @param string $cookie_value
         * @param ?int $cookie_timeout_second
         * @param ?bool $allow_children_domain_use 
         * @param ?bool $allow_children_path_use
         */
        function setCookie(string $n,string $v,?int $t=null,bool $ajsu = false,bool $acdu = false,bool $acpu = true){
            if($this -> state != self::TIMING)
                throw new \Error('The request is not in TIMING state.');
            $e = "$n=$v; ";
            if(!is_null($t)) $e .= "Max-Age=$t; ";  // 失效时间
            if($acdu) $e .= 'Domain='.$this->getrHeader('host').'; ';
            if($acpu) $e .= 'Path=/; ';
            if($ajsu) $e .= 'HttpOnly; ';
            $this->cookies[] = $e;
        }

        /**
         * 获取安全存储区的内容（session）
         * 注意了！不能保证绝对安全
         * 
         * @param string $get_session_name
         * @return mixed
         */
        public function getSession(string $session){
            if($this -> state != self::TIMING)
                throw new \Error('The request is not in TIMING state.');
            $ip = self::getUrl($this -> addr);
            if(is_null($sessid = $this->getCookie('__token'))) return null;
            return self::_getSession($this->getrHeader('host') . "/{$ip}/{$sessid}_{$session}");
        }

        /**
         * 写入安全存储区内容
         * 
         * @param string $set_session_name
         * @param mixed $value
         * @return void
         */
        public function setSession(string $session,$value){
            if($this -> state != self::TIMING)
                throw new \Error('The request is not in TIMING state.');
            $ip = self::getUrl($this -> addr);
            if(is_null($sessid = $this->getCookie('__token'))) 
                $this->setCookie('__token',$sessid = md5(rand(0,99999999)),acpu:true);
            self::_setSession($this->getrHeader('host') . "/{$ip}/{$sessid}_{$session}",$value);
        }

        /**
         * 缓存一些数据
         * 
         * @param string $str
         */
        function write(string $str){
            if($this -> state == self::WEBSOCKET || !is_array($this -> headers))
                $this -> write_buffer .= $str;
            else $this -> temp .= $str;
            return $this;
        }
        
        /**
         * 输出一个文件。若指定offset为true将允许客户端断点续传
         * 【警告】若使用TEST模式，限速只会大大降低并发。
         * 不过可以减少带宽太小导致缓冲区挤满，可以设置为服务器的实际带宽
         * 
         * @param string $file_path
         * @param string $use_offset
         * @return bool
         */
        function file(string $file_name,bool $offset = true){
            if($this -> state != self::TIMING)
                throw new \Error('The request is not in TIMING state.');
            if(!is_file($file_name) or !is_readable($file_name)){
                $this->finish(404,'File not exists!');
                return false;
            }

            // 先决定文件类型等
            $name = basename($file_name);       // 实际名称
            $ext  = $this->getMime($name);      // MIME类型
            $this -> header('Last-Modified', gmdate('D, d M Y H:i:s T',filemtime($file_name)));
            $this -> header('Content-Type',$ext);

            // 判断是否客户端是最新的文件，是的话返回304告诉客户端不用传了
            if($offset and !is_null($this->getrHeader('if-modified-since'))){
                $uv = strtotime($this->getHeader('if-modified-since')); 
                if($uv == filemtime($file_name))
                    $this->finish(304);
            }
            
            // 决定是否内联，如果参数里内置download则直接下载
            $type = $this->getParam('download') ? 'attachment' : 'inline';
            $this -> header('Content-Transfer-Encoding','binary');
            $this -> header('Content-Disposition',"$type;filename=$name");

            $f = Core::open($file_name,'rb');
            $fsize = filesize($file_name);
            
            if($offset === true && $this->getrHeader('range')) {
                $range = $this->getrHeader('range');
                $start = (int)$range;       // 起始位置
                $end   = (int)substr($range,strpos('-',$range)+1);

                if($end == 0)
                    $end = filesize($file_name) -1;
                elseif($end > $fsize)
                    return $this -> finish(416,"Out of fileSize($fsize)");
                elseif($end <= $start)
                    return $this -> finish(400,"Illegal range(#0:$start >= #1:$end)");

                $read_length = $end - $start;
                $this -> header('Content-Range', "bytes $start-$end/$fsize");
                $this -> header('Content-Length',$read_length +1);

                $this->status(206);         // 开始断点续传
                $f -> seek( $start );
            }else{
                $read_length = $fsize;
                $this -> header('Content-Length',$fsize);
            }

            $this -> mkHeader();
            yield $f -> pipeTo($this,$read_length);

            $f -> close();
            return $this -> state == self::CREATED;
        }

        /**
         * 针对HTML压缩减小体积，比GZ压缩更快且更有效
         * 
         * @param string $will_compressed_string
         * @param int $http_status_code
         * @return void
         */
        function useCompress(string $str):void{
            $this->write(preg_replace([
                    // 过滤标记中空格    过滤HTML注释       过滤JS注释            过滤CSS注释
                    '/> *([^ ]*) *</', '/<!--[^!]*-->/', '/^"?\\/\\/[^\n]+/' , '/\\/\\*[^(\*\/)]+\\*\\//','/[\s]+/'   
                ],[ '>\\1<'          , ''              ,''                   ,''                     , ' '],$str)
            );
        }

        /**
         * 列举文件，使用内置CSS
         * @param string 列举文件夹
         * @param bool 是否有父目录可列举
         * @param string URL相对路径，用于链接
         */
        function listDir(string $dir,bool $parent=false,string $relative_path = '.'){
            if($this -> state != self::TIMING)
                throw new \Error('The request is not in TIMING state.');
            if(false === $data = scandir($dir))
                return $this -> finish(403,'Failed to open dir');
            $this -> write('<html><head><meta http-equiv="content-type" content="text/html; charset=UTF-8"><title>WebList</title><style>');
            $this -> write(preg_replace('/\\s+/',' ',self::$css));
            $this -> write('</style></head><body><h2> WebList </h2><div><h><a>名称</a><c>大小</c><c>最后更新</c></h>');
            if($parent)
                $this -> write('<d><a href="../">上一级目录</a><c>-</c><c>-</c></d>');
            $file = '';
            foreach ($data as $fname) {
                $path = "$dir/$fname";
                if($fname[0] == '.') continue;
                // 文件夹
                if(is_dir($path))
                    $this -> write("\n<d><a href=\"$relative_path/$fname/\">$fname/</a><c>-</c><c>-</c></d>");
                // 文件
                else{
                    try{
                        $stat = @lstat($path);
                    }catch(\Throwable){
                        $stat = [
                            "size"  => -1,
                            "mtime" => 0
                        ];
                    }
                    if(!$stat) continue;
                    if($stat['mtime'] > 0)
                        $date = date('Y-m-d H:M',$stat['mtime']);
                    else $date = '-';
                    $size = '-';
                    if($stat['size'] >= 0)
                        foreach (['B','KB','MB','GB','TB','PB'] as $i => $unit)
                            if($stat['size'] / 1024 ** $i < 100){
                                $size = round($stat['size'] / 1024 ** $i,1) . $unit;
                                break;
                            }
                    $file .= "\n<f><a href=\"$relative_path/$fname\">$fname</a><c>$size</c><c>$date</c></f>";
                }
            }
            $this -> write($file);
            $this -> finish(200,'</div><span> EvPHP WebList </span></body></html>');
        }

        /**
         * 变成WebSocket请求
         */
        function ws():Promise{
            if($this -> state != self::TIMING)
                throw new \Error('The request is not in TIMING state.');
            return Promise::new(function($rs,$rj){
                $key = $this -> getHeader('sec-websocket-key');
                if(\trim(\strtolower($this -> getrHeader('Upgrade'))) != 'websocket' && !$key)
                    return $rj(new \Error('Error: Not a WebSocket request.'));

                $accept = \base64_encode(\pack('H*', 
                    \sha1($key . '258EAFA5-E914-47DA-95CA-C5AB0DC85B11')
                ));
                $this -> mkHeader(101,[
                    "Upgrade" => "websocket",
                    "Connection" => "Upgrade",
                    "Sec-WebSocket-Version" => "13",
                    "Sec-WebSocket-Accept" => "$accept"
                ]);
                $this -> state = self::WEBSOCKET;

                @ob_clean();
                $rs(new WebSocket($this));
            });
        }

        /**
         * 使用内置WebDAV服务器
         * 注意：应该先身份认证
         */
        function webdav(){
            $path = self::$dav_root . $this -> path;

            if($this -> state != self::TIMING)
                throw new \Error('The request is not in TIMING state.');

            $get_dest = function() use ($path){
                $dest = $this -> getrHeader('destination');
                if($dest[0] == '/')
                    return self::$dav_root . $dest;
                elseif(is_dir($path))
                    return $path . '/' . $dest;
                else
                    return dirname($path) . '/' . $dest;
            };

            switch (strtoupper($this -> method)) {
                case 'GET':
                    yield $this -> file($path);
                break;

                case 'PUT':
                    if (!$this->getrHeader('content-length'))
                        return $this->finish(400, "No Length specificed");

                    try{
                        $f = Core::open($path, 'w');
                    }catch(\Throwable){
                        return $this -> error(409);
                    }

                    if ($f === false) 
                        return $this -> error(403, 'OpenFile failed.');

                    $read_length = (int)$this->getrHeader('content-length');
                    $_100 = strtolower(trim($this -> getrHeader('expect'))) == '100-continue';

                    if($_100) $this -> finish(100);
                    yield $this -> pipeTo($f,$read_length);
                    
                    if(!$_100) $this -> finish(201);
                    else $this -> close();
                break;

                case 'DELETE':
                    if(@unlink($path))
                        $this->finish(204);
                    else $this->finish(424);
                break;

                case 'MOVE':
                    $d = $get_dest();
                    if(file_exists($d) && strtoupper(trim($this -> getrHeader('overwrite'))) != 'T')
                        $this -> finish(412);
                    else
                        $this -> finish(@rename($path,$d) ? 201 : 403);
                break;

                case 'COPY':
                    /**
                     * @todo 异步的copy
                     */
                    @copy($path,$get_dest());
                break;

                case 'MKCOL':
                    if(file_exists($path))
                        $this -> finish(405);
                    elseif(!is_dir(dirname($path)))
                        $this -> finish(409);
                    elseif(@mkdir($path))
                        $this->finish(201);
                    else
                        $this->finish(403);
                break;

                case 'PROPFIND':
                    $this -> header('Content-Type','application/xml');
                    $this -> write('<?xml version="1.0" encoding="utf-8" ?>');
                    $this -> write('<D:multistatus xmlns:D="DAV:">');
                    $this -> _propfind(
                        $path,
                        $this -> path,
                        (int)$this->getrHeader('depth')
                    );
                    $this -> finish(200,'</D:multistatus>');
                break;
                
                case 'PROPPATCH':
                    $this -> finish(200);
                break;

                default:
                    $this -> finish(405,'Unknown method '.$this -> method);
                break;
            }
        }

        private function _propfind(string $path,string $relative_uri,int $depth_rest){
            $data = @lstat($path);
            $path = htmlspecialchars($path,ENT_XML1);
            $relative_uri = preg_replace('/(?:\/|\\\\)+/','/',$relative_uri) ?? '/';

            if(is_dir($path)){
                $this -> useCompress("<D:response>
                    <D:href>$relative_uri</D:href>
                    <D:propstat>
                        <D:prop>
                            <D:resourcetype><D:collection/></D:resourcetype>
                            <D:getlastmodified>" . date('D, j M Y H:i:s \G\M\T', $data['mtime']) . "</D:getlastmodified>
                            <D:creationdate>" . date('Y-m-d\TH:i:s\Z', $data['ctime']) . "</D:creationdate>
                        </D:prop>
                        <D:status>HTTP/1.1 200 OK</D:status>
                    </D:propstat>
                    <D:propstat>
                        <D:prop>
                            <D:getcontentlength/>
                        </D:prop>
                        <D:status>HTTP/1.1 404 Not Found</D:status>
                    </D:propstat>
                </D:response>");
                // 扫描子目录
                if($depth_rest > 0)
                    foreach (scandir($path) as $item)
                        if($item != '.' && $item != '..')
                            $this -> _propfind($path . '/' . $item, $relative_uri . '/' . $item,$depth_rest -1);
            }else{
                $this -> useCompress("<D:response>
                    <D:href>$relative_uri</D:href>
                    <D:propstat>
                        <D:prop>
                            <D:getlastmodified>" . date('D, j M Y H:i:s \G\M\T', $data['mtime']) . "</D:getlastmodified>
                            <D:creationdate>" . date('Y-m-d\TH:i:s\Z', $data['ctime']) . "</D:creationdate>
                            <D:getcontentlength>" . $data['size'] . "</D:getcontentlength>
                            <D:resourcetype/>
                        </D:prop>
                        <D:status>HTTP/1.1 200 OK</D:status>
                    </D:propstat>
                </D:response>");
            }
        }
    }

    class WebSocket{

        /**
         * @var HttpHandle 基准TcpHandle
         */
        private $handle;

        /**
         * @var callable 关闭连接时的回调
         */
        public $onclose;

        /**
         * @var callable PONG时的回调
         */
        public $onpong;

        const MAX_BODY = 1 * 1024 * 1024;

        public function __construct(HttpHandle $handle) {
            $this -> handle = $handle;
            Core::log('{color_blue}N{/} New WebSocket Client');
        }

        /**
         * 向远程写入数据
         */
        public function send(string $content){
            // 是否是最后一帧
            $this -> handle -> write(chr(0x80 | 1));

            // 有效负载数据的长度
            $len = strlen($content);
            if ($len > 0 && $len <= 125)
                $this -> handle -> write(chr(0 | $len));
            elseif ($len <= 65535)
                $this -> handle -> write( chr(0 | 126) . chr($len >> 8 ) . chr($len & 0xff));
            else
                $this -> handle -> write(\chr(0 | 127)
                    . \chr($len >> 56)
                    . \chr($len >> 48)
                    . \chr($len >> 40)
                    . \chr($len >> 32)
                    . \chr($len >> 24)
                    . \chr($len >> 16)
                    . \chr($len >> 8)
                    . \chr($len >> 0)
                );
            $this -> handle -> write($content);
        }

        public function onMessage(callable $cb){
            while(true)
                if($data = yield $this -> read())
                    $cb($data);
        }

        /**
         * 读取客户端数据
         * 注意：这是异步函数，使用yield + Promise::await才能运行
         * 
         * @link https://github.com/walkor/workerman/blob/master/src/Protocols/Websocket.php
         * @link https://apifox.com/apiskills/websocket-protocol/
         */
        public function read():\Generator{
            // 读取头
            $buffer = yield $this -> handle -> read(6);
            $firstByte = ord($buffer[0]);
            $secondByte = ord($buffer[1]);
            // 解析数据
            $dataLen = $secondByte & 127;
            $isFinFrame = $firstByte >> 7;
            $masked = $secondByte >> 7;
            $opcode = $firstByte & 0xf;

            // 解析内容长度
            if ($dataLen <= 125)
                // 数据长度: $dataLen
                $len = $dataLen;
            elseif ($dataLen == 126){
                // 数据长度: 读取2位
                $buffer = yield $this -> handle -> read(2);
                $len = unpack('nlen', $buffer)['len'];
            }elseif ($dataLen == 127){
                // 数据长度: 读取8位
                $buffer = yield $this -> handle -> read(8);
                $upk = unpack('N2len', $buffer);
                $len = $upk['len1'] * 4294967296 + $upk['len2'];
            }
            
            // 超过限制长度
            if($len > self::MAX_BODY)
                return $this -> handle -> close();

            // 读取掩码
            if($masked){
                $mask = yield $this -> handle -> read(4);
                // 符合长度
                $masks = str_repeat($mask, floor($len / 4)) . substr($mask, 0, $len % 4);
            }

            // 读取数据
            $readed = 0;
            if($masked){
                $result = '';
                do{
                    $rest = $len - $readed;
                    $read = $rest > 4 ? 4 : $rest;
                    $data = yield $this -> handle -> read($read);
                    if($read < 4) $masks = substr($masks,0,$read);
                    $result .= $data ^ $masks;
                    $readed += $read;
                }while($readed < $len);
            }else{
                $result = yield $this -> handle -> read($len);
            }

            // 解析OPCode
            switch ($opcode) {
                // ===== 数据帧 =====
                // 连续帧
                case 0x0:
                // 文本帧
                case 0x1:
                // 二进制帧
                case 0x2:
                break;
                
                // ===== 功能帧 =====
                // PING
                case 0x9:
                    if (!$isFinFrame)
                        return $this -> handle -> close();
                    $this -> handle -> write($data);
                break;

                // PONG
                case 0xa:
                    if(is_callable($this -> onpong))
                        call_user_func($this -> onpong,$data);
                break;

                // 连接断开
                case 0x8:
                    if(is_callable($this -> onclose))
                        call_user_func($this -> onclose);
                    // 关闭连接
                    $this -> handle -> write("\x88\x02\x03\xe8");
                    $this -> handle -> close();
                break;

                // Wrong opcode.
                default :
                    Core::log('{color_red}E{/} ClientError: Unknown websocket opcode');
                    $this -> handle -> close();
                break;
            }

            if($isFinFrame)
                return $data;
            else
                return $data . $this -> read(); 
        }
    }

    /**
     * 文件处理类，此可以轻松替换rename类函数
     */
    class FileObject{
        public 
            $data,  // 文件数据
            $name,  // 文件名称
            $type,  // 文件类型(MIME)
            $form;  // 在form中它的名称

        /**
         * 解析内容
         * 
         * @param string $file 要保存的原生POST内容
         */
        function __construct(string $file){
            list($head,$this->data) = explode("\r\n\r\n",$file,2);
            $tmp = [];              // 读取头部
            foreach(explode(PHP_EOL,$head) as $n){
                if($n == '') return;
                list($n,$v) = explode(':',$n);
                $tmp[strtolower(trim($n))] = trim($v);
            }
            $this->type = $tmp['content-type']??'text/plain';// 如果是文本，没有MIME
            list($type,$name,$fname) = explode(';',$tmp['content-disposition']);
            if(is_null($fname) or is_null($name)) return;   // 错误的文件
            $this->name = trim(explode('=',$fname,2)[1]);
            $this->form = trim(explode('=',$name,2)[1]);
        }

        /**
         * 保存文件
         */
        function save(string $path,bool $forse = false):int{
            // if(is_dir(dirname($path))) if($forse) mkdir(dirname($param)); else return 1;
            if(file_exists($path) and !$forse) return 2;
            if(false == file_put_contents($path,$this->data)) return 3;
            return 0;
        }
        
    }

    function __main(){
        $cfile = "config.json";
        $config = [
            "server" => [
                "bind" => [
                    "tcp://0.0.0.0:80"
                ],
                "ssl" => [
                    "enable" => false,
                    "cert" => "",
                    "key" => ""
                ]
            ],
            "rounger" => [
                "execable" => [
                    "php"
                ],
                "index" => [
                    "index.php",
                    "index.html"
                ],
                "autoindex" => true,
                "root" => __DIR__
            ],
            "behavior" => [
                "displayWarning" => true
            ]
        ];

        if(is_file($cfile)){
            $data = @json_decode(file_get_contents($cfile),true);
            if(!is_array($data))
                throw new \Error("Failed to load config.");
            $config = array_merge($config,$data);
        }

        define('CONFIG',$config);

        foreach (CONFIG['server']["bind"] as $addr)
            Core::bind($addr);

        HttpHandle::$dav_root = CONFIG['base'];

        Core::start();
    }

    // 设置Error handle
    set_error_handler(function(int $errno,string $errstr,string $errfile,int $errline){
        if($errno == E_ERROR)
            Core::log("{color_red}E{/} {font_bold}UncaughtError{/}({color_gray}in $errfile:$errline{/}): $errstr");
        elseif(CONFIG['behavior']['displayWarning'])
            Core::log("{color_blue}W{/} $errstr {color_gray}in $errfile:$errline{/}");
        return true;
    });

    // Start the project
    if(!defined('__MAIN__')){
        define('__MAIN__',__FILE__);
        __main();
    }
} ?>