#!/usr/data/php
<?php
    namespace Timer;
    use MomentCore\Core;
    use MomentCore\Handler;
    use MomentCore\Promise;
    use stdClass;

    if(defined('__MAIN__')) return function(\MomentCore\HttpHandle $handle){
?>

<!DOCTYPE html>
<html>
    <head>
        <title>Tasker</title>
        <meta charset="UTF-8">
        <style>
            html{
                background: linear-gradient(45deg, #6fe7e0, #e772eb);
                height: 100vh;
            }
            body{
                margin: 2rem auto;
                overflow: hidden;
                max-width: 35rem;
                border-radius: 1rem;
                background-color: white;
                box-shadow: 0 0 1rem lightgray;
            }
            .head{
                background-color: rgb(102, 156, 242);
                color: white;
                text-align: center;
                font-size: 2rem;
                padding: 1rem 0;
                border-radius: 1rem 1rem 0 0;
            }
            .body{
                padding: 1rem 2rem 2rem 2rem;
            }
            code{
                background-color: rgb(218, 213, 206);
                border-radius: .2rem;
                padding: .5rem;
                margin: .8rem .2rem;
                display: block;
                white-space-collapse: preserve;
            }
            a.btn {
                background-color: #c74ae3;
                border: solid .1rem transparent;
                color: white;
                padding: .4rem 1.2rem;
                text-decoration: none;
                display: inline-block;
                margin: .2rem 1rem;
                cursor: pointer;
                border-radius: .3rem;
                transition: all .2s;
            }
            .btn[disabled]{
                opacity: .6;
                pointer-events: none;
            }
            .btn:not([disabled]):hover {
                border-color: #c74ae3; /* Green */
                background-color: white;
                color: #c74ae3;
            }
        </style>
    </head>
    <body>
        <?php
            global $units;
            $sname = $handle -> getParam('unit');
            $action = $handle -> getParam('action');
            /**
             * @var ServiceUnit
             */
            $service = @$units[$sname];
        ?>

        <?php if($service): ?>
            <div class="head"
                style="background-color: <?php echo $service -> getStatus()-> running ? 'green' : 'red' ?>;"
            >
                <?php echo $sname; ?>
                <?php $service -> getStatus() -> pid; ?>
            </div>
            <div class="body">
                <?php if($action) try{
                    switch($action){
                        case 'start':
                            $rs = $service -> start();
                        break;
                        
                        case 'stop':
                            $rs = $service -> close();
                        break;
                    }
                    echo '<!--' . $rs . '-->';
                    if($rs == ServiceUnit::OK){ ?>
                        <p style="color: red;text-align: center;">
                            成功执行操作 <?php $action; ?>
                        </p>
                    <?php } ?>
                <?php }catch(\Throwable $e){?>
                    <p>
                        <span style="color: red;">执行操作失败</span>
                        <?php echo Core::parseError($e); ?>
                    </p>
                <?php }finally{ ?>
                    <script>
                        setTimeout(function(){
                            document.location = '?unit=<?php echo $sname ?>';
                        },3000);
                    </script>
                <?php } ?>
                
                <code><?php echo yield $service -> read(); ?></code>
                <div>
                    <a href="?action=shutdown&unit=<?php echo $sname ?>"
                        <?php if($action == 'stop') echo 'disabled'; ?> 
                        class="btn"
                    >停止</a>
                    <a href="?action=start&unit=<?php echo $sname ?>"
                        <?php if($action == 'start') echo 'disabled'; ?> 
                        class="btn"
                    >启动</a>
                    <a onclick="document.location.reload()" class="btn">刷新</a>
                </div>
            <?php else: ?>
                <div class="head">
                    Unit <?php echo $sname; ?>不存在
                </div>
                <div class="body">
                    <?php foreach ($units as $name => $_):?>
                        <div><a href="?unit=<?php echo $name; ?>"><?php echo $name; ?></a></div>
                    <?php endforeach; ?>
                </div>
            <?php endif; ?>
        </div>
    </body>
</html>

<?php }; else{
    define('__MAIN__',__FILE__);
    include_once "server.php";

    Core::$alias['/manage.php'] = __FILE__;
    
    /**
     * 秒级Timer
     */

    $lastTime = time();
    $stack = [];
    $utils = [];

    /**
     * Example
     * 1 => [],
     * 3 => []
     */

    Core::$onfree[] = function() use (&$lastTime,&$stack){
        $after = time();
        // 过了1s
        if($after > $lastTime){
            $lastTime = $after;
            foreach($stack as $time => &$object){
                // 增加1s
                $object['interval'] ++;
                // 达到指定值
                if($object['interval'] >= $time){
                    $object['interval'] = 0;
                    // 执行回调
                    foreach ($object as $key => $value) {
                        if(is_numeric($key)) try{
                            Promise::await($value);
                        }catch(\Throwable){}
                    }
                }
            }
        }
    };

    function interval(callable $callback,int $freq_time = 1){
        global $stack;

        if($freq_time <= 0)
            throw new \TypeError('Time <= 0 is illegal');
        if(!array_key_exists($freq_time,$stack))
            $stack[$freq_time] = [
                'interval' => $freq_time
            ];
        $stack[$freq_time][] = $callback;
    }

    class ServiceUnit extends Handler{
        static $watch = [];

        readonly object $config;

        private $proc;
        private $stderr;
        private $stdout;
        public $exit_code = -1;

        const ERR_IS_RUNNING = 1;
        const ERR_NOT_RUNNING = 2;
        const ERR_NOT_FOUND = 3;
        const ERR_EXEC_FAILED = 4;
        const ERR_NOT_INITED = 10;
        const OK = 0;

        const BUFFER_LEN = 4 * 1024;

        static function feed(){
            foreach (self::$watch as $unit)
                $unit -> watch_feed();
        }

        public function __construct(string $path) {
            try{
                $file = file_get_contents($path);
                $config = json_decode($file);
                if(!$config)
                    throw new \Error("Failed to parse config.");
                $this -> config = $config;

                $this -> config -> source = $path;
                $this -> config -> name = strtolower(substr(basename($path),0,strlen(".json")));
                if(!is_array($this -> config -> args))
                    throw new \TypeError("StartArgument(args) not found in UNIT:{$this -> config -> source}",3);

                if($this -> config -> watch)
                    self::$watch[] = $this;
            }catch(\Throwable $e){
                $fname = basename($path);
                $line = $e -> getLine();
                Core::log("{color_red}E {color_gray}{$fname}{/} init failed [$line] ({color_red}" . $e->getMessage() . "{/})");
                $this -> config = new \stdClass();
            }
        }

        public function start(){
            if(!$this -> config -> source)
                return self::ERR_NOT_INITED;
            if($this -> getStatus()-> running)
                return self::ERR_IS_RUNNING;

            $args = $this -> config -> args;
            
            if($this -> config -> workdir && !is_dir($this -> config -> workdir))
                Core::log("{color_yellow}W{/} WorkingDirectory not found in UNIT:{$this -> config -> name}");

            try{
                $this -> proc = proc_open($args,[
                    1 => ['pipe','w'],
                    2 => ['pipe','w']
                ],$pipe,$this -> config -> workdir,[
                    "PARENT_PID" => getmypid()
                ]);
                if(!$this -> proc)
                    throw new \Exception(error_get_last()['message']);

                Core::_add($pipe[1],$this);
                Core::_add($pipe[2],$this);
                $this -> stdout = $this -> socket = $pipe[1];
                $this -> stderr = $pipe[2];

                $this -> exit_code = -1;
            }catch(\Error $e){
                Core::log("{color_red}E{/} Exec failed: " . $e->getMessage() . " :in UNIT:{$this -> config -> source}");
                return self::ERR_EXEC_FAILED;
            }

            return self::OK;
        }

        public function close():void{
            if(-1 != $this -> exit_code && $this -> proc)
                return;
            $this -> exit_code = proc_close($this -> proc);
        }

        public function closed(): bool{
            return $this -> getStatus() -> running;
        }

        public function read(int $read = -1):Promise{
            return Promise::new(fn ($rs) => $rs($this -> read_buffer));
        }

        public function watch_feed(){
            if(-1 == $this -> exit_code && !$this -> closed()) return self::OK;
            Core::log("{color_green}I{/} Service {$this -> config -> name} stopped.restarting");
            $this -> start();
        }

        public function getStatus(){
            if(!$this -> proc){
                $data = new stdClass();
                $data -> running = false;
                return $data;
            }
            $status = (object)proc_get_status($this -> proc);
            if($status -> stopped)
                if($status -> exitcode != -1)
                    $this -> exit_code = $status -> exitcode;
                else
                    $status -> exitcode = $this -> exit_code;
            return $status;
        }

        public function _read(){
            $data = fread($this -> stdout,self::BUFFER_LEN) . fread($this -> stderr,self::BUFFER_LEN);
            $t = $this -> read_buffer . $data;
            if(strlen($t) > self::BUFFER_LEN)
                $this -> read_buffer = substr($t,strlen($t) - self::BUFFER_LEN);
            else
                $this -> read_buffer = $t;
        }
    }

    // php task
    foreach (glob('*.task.php') as $php) {
        try{
            include $php;
        }catch(\Throwable){
            Core::log('{color_red}E{/} Error while import '.$php);
        }
    }

    // json 服务
    foreach (glob('*.task.json') as $file) {
        $unit = new ServiceUnit($file);
        $units[$unit -> config -> name ?? uniqid('s_')] = $unit;
        try{
            $unit -> start();
        }catch(\Throwable){}
    }

    // 值守
    interval(fn() => ServiceUnit::feed(),10);

    // 启动EventLoop服务器
    \MomentCore\__main();
} ?>